package ch.hevs.kart;

import java.security.InvalidParameterException;

/**
 * Enumerates the different status registers that a kart might send to the smart-phone.
 *
 * This class implements the protocol/register map specification from
 * <a href="http://wiki.hevs.ch/fsi/index.php5/Kart/serial_link">here</a>
 *
 * @see KartStatusRegisterListener
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.2 - 2016
 */
public enum KartStatusRegister {
	/**
	 * Hall sensor 1 pulse count.
	 * <br><br>
	 * This register counts the pulses on the input of the first hall effect sensor input. The input
     * connector for the sensor is labeled <b>IN3</b> on the sensor I/O board. The counter continues
     * counting pulses until it overflows. The maximal value the register can hold is 65'535, then
     * it restarts counting at 0.
	 */
	HallSensorCounter1(0),
	
	/**
	 * Hall sensor 2 pulse count.
     * <br><br>
     * This register counts the pulses on the input of the second hall effect sensor input. The
     * input connector for the sensor is labeled <b>IN4</b> on the sensor I/O board. The counter
     * continues counting pulses until it overflows. The maximal value the register can hold is
     * 65'535, then it restarts counting at 0.
	 */
	HallSensorCounter2(1),
	
	/**
	 * Steering FSM status.
     * <br><br>
     * Reflects the actual status of the on-board steering finite state machine.
	 * <br><br>
	 * <b>Bit 0:</b><br>
	 * If bit 0 is 1, the stepper end position switch has been triggered, if 0 the switch is open.
	 *
	 * <b>Bit 1:</b><br>
	 * A value of 0 means that the steering FSM is actually moving to the target position, a value
	 * of 1 means that the target position has been reached.
	 */
	SteeringStatus(2),

	/**
	 * Actual steering position.
     * <br><br>
     * Reflects the actual steering position of the steering stepper motor control. The minimal
     * position is 0, whereas this can be depending on the kart's steering system and geometry the
     * left or right position of the steering. The center and maximal value varies between karts and
     * depends the steering system geometry too. A reasonable value for the center position is at
     * between 250 and 500, for the end position between 500 and 1000 respective.
	 */
	SteeringPosition(3),
	
	/**
	 * State of the <b>external</b> steering end switch.
     * <br><br>
     * Reflects the actual state of the auxiliary steering end position detection switch. This
     * switch can be connected to the I/O module using the connector with the label <b>IN2</b>.
     * A value of 1 indicates that the switch is open. A value of 0 indicates that the end switch is
     * closed.
     * <br><br>
     * Note that this register does not reflect the state of the main steering end position switch!
	 */
	ExternalEndSwitchState(4),
	
	/**
	 * ADC value of the battery voltage level measure.
     * <br><br>
     * Raw ADC (Analog to Digital Converter) output value. We refer you to the schematic of the
     * power module, the data-sheet of the used
     * <a href="http://wiki.hevs.ch/fsi/images/5/50/MCP3425.pdf">MCP3425</a> ADC and the
     * information you find about the used battery packs in order to calculate the battery charge in
     * function to the ADC value.
     * <br><br>
     * A simpler solution might be to use the {@link KartListener} interface, it returns the
     * normalized battery level from 0 (empty) to 1 (full).
	 */
	BatteryVoltageLevel(5),
	
	/**
	 * Ultrasonic distance ranger value.
     * <br><br>
     * This status register reflects the actual measured value of by using the ultrasonic distance
     * ranger from <a href="http://www.elecfreaks.com/store/hcsr04-ultrasonic-sensor-distance-measuring-module-ultra01-p-91.html">
     * here</a>. The sensor has to be connected to the connector labeled <b>IN1</b> on the
     * <br><br>
     * The distance measure block in the sensors FPGA counts the ultrasonic pulse length in terms
     * of the 10 MHz clock periods. The distance im meters (m) is thus given by
     * <b>distance = count * 0.000017</b> or for millimeters (mm) <b>distance = count * 0.017</b>.
     * The register has a size of 16 bites, so values between 0 and 65353 and thus distances between
     * <b>0m</b> and <b>1.11m</b> can be measured.
	 */
	DistanceSensor(6),

    /**
     * Embedded sequence player status.
     * <br><br>
     * <b>EXPERIMENTAL: This feature is experimental and might not be available on all Karts!</b>
     * <br><br>
     * Reflects the actual state of the embedded hardware sequence player on the kart. A value of
     * 1 means that the sequence is actually running, a value of 0 means that no sequence is
     * actually running.
     */
    SequenceStatus(7),

	/**
	 * Proximity sensor 1 value.
     * <br><br>
     * Contains the actual value of the IR proximity sensor
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the first I2C input with the label <b>I2C1</b>. We refer to the data-sheet of the sensor in
     * order to calculate the distance from the value.
	 */
	ProximitySensor1(11),

    /**
     * Proximity sensor 2 value.
     * <br><br>
     * Contains the actual value of the IR proximity sensor
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the second I2C input with the label <b>I2C2</b>. We refer to the data-sheet of the sensor in
     * order to calculate the distance from the value.
     */
	ProximitySensor2(10),

    /**
     * Proximity sensor 3 value.
     * <br><br>
     * Contains the actual value of the IR proximity sensor
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the third I2C input with the label <b>I2C3</b>. We refer to the data-sheet of the sensor in
     * order to calculate the distance from the value.
     */
	ProximitySensor3(9),

    /**
     * Proximity sensor 4 value.
     * <br><br>
     * Contains the actual value of the IR proximity sensor
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the fourth I2C input with the label <b>I2C4</b>. We refer to the data-sheet of the sensor in
     * order to calculate the distance from the value.
     */
	ProximitySensor4(8),

    /**
     * Proximity sensor 1 ambient light measure.
     * <br><br>
     * Contains the actual value of the IR proximity sensor's ambient light measure
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the first I2C input with the label <b>I2C1</b>. We refer to the data-sheet of the sensor in
     * order to interpret the value.
     */
	AmbientLightSensor1(15),

    /**
     * Proximity sensor 2 ambient light measure.
     * <br><br>
     * Contains the actual value of the IR proximity sensor's ambient light measure
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the second I2C input with the label <b>I2C2</b>. We refer to the data-sheet of the sensor in
     * order to interpret the value.
     */
	AmbientLightSensor2(14),

    /**
     * Proximity sensor 3 ambient light measure.
     * <br><br>
     * Contains the actual value of the IR proximity sensor's ambient light measure
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the third I2C input with the label <b>I2C3</b>. We refer to the data-sheet of the sensor in
     * order to interpret the value.
     */
	AmbientLightSensor3(13),

    /**
     * Proximity sensor 4 ambient light measure.
     * <br><br>
     * Contains the actual value of the IR proximity sensor's ambient light measure
     * <a href="http://www.vishay.com/docs/83798/vcnl4000.pdf">VCNL4000</a> which is connected to
     * the fourth I2C input with the label <b>I2C4</b>. We refer to the data-sheet of the sensor in
     * order to interpret the value.
     */
	AmbientLightSensor4(12);
		
	KartStatusRegister(final int address) {
		this.address = address;
	}
	
	/**
	 * Returns the actual address of the status register in the Kart's register memory. The register
     * map is according to the documentation
     * <a href="http://wiki.hevs.ch/fsi/index.php5/Kart/serial_link">here</a>.
	 *
	 * @return Status register address.
	 */
	public int getAddress() {
		return address;
	}
	
	/**
     * Creates a status register enumerator from the given register address. The register
     * map is according to the documentation
     * <a href="http://wiki.hevs.ch/fsi/index.php5/Kart/serial_link">here</a>.
     *
     * @param address                       The address of the register in the kart's register
     *                                      memory.
     * @return                              The address enumeration.
     * @throws InvalidParameterException    If no register is defined for the given address.
     */
	public static KartStatusRegister fromAddress(final int address)
            throws InvalidParameterException {
		for (KartStatusRegister register: values()) {
			if (address == register.getAddress())
				return register;
		}
		throw new InvalidParameterException("No KartStatusRegister with address " + address +
                " defined!");
	}

    // The total number of status registers.
    static final int COUNT = 16;

	// The address of the register enumeration.
	private final int address;
}
